from django.http import HttpRequest, HttpResponse
from django.shortcuts import redirect

__all__ = ["RedirectHelper"]


class RedirectHelper(object):
    """
    重定向辅助
    """

    @staticmethod
    def to_url(request: HttpRequest, default_url: str, next_param: str = "next") -> str:
        """
        获取需要跳转到的 URL
        """
        # noinspection PyTypeChecker
        gn = request.GET.get(next_param, None)
        if gn is not None:
            return gn

        # noinspection PyTypeChecker
        pn = request.POST.get(next_param, None)
        if pn is not None:
            return pn

        return default_url

    @staticmethod
    def to_page(
        request: HttpRequest, default_url: str, next_param: str = "next"
    ) -> HttpResponse:
        """
        跳转到 request 指定的页面 或者 默认的页面
        """
        url = RedirectHelper.to_url(request, default_url, next_param)
        return redirect(url)



"""
使用方式:

在配置文件中:
```python
from django_qiyu_utils.settings import *
```
"""

import inspect
import os

from django.urls import path
from django.views.static import serve

from .env import EnvHelper

# 只需要导出有用的变量
#
# Django 项目导出的配置项
#
__all__ = [
    "DEBUG",  # by env
    "SECRET_KEY",  # by env
    "ALLOWED_HOSTS",  # by env
    "LANGUAGE_CODE",  # by env
    "TIME_ZONE",  # by env
    "MEDIA_URL",  # by env
    "MEDIA_ROOT",  # by env
    "STATIC_URL",  # by env
    "STATIC_ROOT",  # by env
    "STATICFILES_DIRS",
    # 静态文件服务
    "SERVE_FILE_URLS",  # by env
]

# 安全警告: 请不要在线上环境打开 DEBUG
DEBUG = False  # 防御性编程 默认不打开 DEBUG
# 只有设置了 DJANGO_DEV|DJANGO_TEST 环境变量, 才打开 DEBUG
if EnvHelper.in_dev() or EnvHelper.in_test():
    DEBUG = True

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/3.1/howto/deployment/checklist/
#
# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = os.getenv(
    "DJANGO_SECRET_KEY", "wr*dd-zawlb)xpzw!hi-jbkene_^sb7oqs4_(e7Tz8yklz49_v"
)

#
# 允许访问的 HOSTS 列表
ALLOWED_HOSTS = ["*"]
# 多个域名使用 ';' 分隔
hosts = os.getenv("DJANGO_ALLOWED_HOSTS", None)
if isinstance(hosts, str) and hosts != "":
    ALLOWED_HOSTS = hosts.split(";")
##########################################################

LANGUAGE_CODE = os.getenv("DJANGO_LANGUAGE_CODE", "zh-Hans")

TIME_ZONE = os.getenv("DJANGO_TIME_ZONE", "Asia/Shanghai")

MEDIA_URL = os.getenv("DJANGO_MEDIA_URL", "/media/")

STATIC_URL = os.getenv("DJANGO_STATIC_URL", "/static/")

#################################################################################
# this is ugly hack
frame = inspect.currentframe()
BASE_DIR = os.path.normpath(
    os.path.abspath(os.path.dirname(inspect.getfile(frame.f_back)))
)

MEDIA_ROOT = os.path.normpath(
    os.getenv("DJANGO_MEDIA_ROOT", os.path.join(BASE_DIR, "..", "media"))
)
STATIC_ROOT = os.path.normpath(
    os.getenv("DJANGO_STATIC_ROOT", os.path.join(BASE_DIR, "..", "static"))
)

STATICFILES_DIRS = []
if DEBUG:
    STATICFILES_DIRS = [STATIC_ROOT]
    # this do fix the problem
    # ?: (staticfiles.E002) The STATICFILES_DIRS setting
    # should not contain the STATIC_ROOT setting.
    STATIC_ROOT = None
#################################################################################


#################################################################################
# 静态文件服务 url
#
# 友情提示:
# 如果您的网站访问量比较大，那么应该使用 Web 服务器来托管静态文件
# 如果你的网站访问量非常大，那么应该使用 CDN 来托管静态文件
#
SERVE_FILE_URLS = []
# SERVE_STATIC_FILES
if os.getenv("DJANGO_SERVE_STATIC_FILES", None) is not None:
    SERVE_FILE_URLS += [path(STATIC_URL, serve, {"document_root": STATIC_ROOT})]
# SERVE_MEDIA_FILES
if os.getenv("DJANGO_SERVE_MEDIA_FILES", None) is not None:
    SERVE_FILE_URLS += [path(MEDIA_URL, serve, {"document_root": MEDIA_ROOT})]
#################################################################################



from typing import Callable

from django.template.context import Context
from django.template.defaulttags import Node, NodeList

__all__ = ["DivNode"]


class DivNode(Node):
    """
    Django custom node like html div
    with function `fn` indicate if should render the inner `nodes`
    """

    def __init__(self, name: str, node_list: NodeList, fn: Callable[[Context], bool]):
        """
        :param name:  the name of the node [for debug and django inner use]
        :param node_list: the inner node list
                          [if `fn` return true then render the inner node otherwise render nothing]
        :param fn: the function to indicate if need render the inner code
                   accept the `Context` as argument, return bool
        """
        self._name = name
        self._node_list = node_list
        self._fn = fn

    def __str__(self):
        return f"<{self._name}>"

    def render(self, context: Context):
        if self._fn(context):
            return self._node_list.render(context)
        return ""



import os
import sys
from typing import Optional

__all__ = ["EnvHelper"]


class EnvKeys(object):
    """
    所有支持的 环境变量 名称
    """

    django_prod = "DJANGO_PROD"  # 如果存在这个环境变量, 则认为是在 生产 环境运行
    django_test = "DJANGO_TEST"  # 如果存在这个环境变量, 则认为是在 测试 环境运行
    django_dev = "DJANGO_DEV"  # 如果存在这个环境变量, 则认为是在 开发 环境运行


class EnvHelper(object):
    """
    当前运行环境的辅助类
    """

    @staticmethod
    def in_prod() -> bool:
        """
        是否运行在线上环境
        """
        return EnvHelper._check_env_exists(EnvKeys.django_prod)

    @staticmethod
    def in_test() -> bool:
        """
        是否运行在测试环境
        """
        return EnvHelper._check_env_exists(EnvKeys.django_test)

    @staticmethod
    def in_dev() -> bool:
        """
        是否运行在开发环境
        """
        return EnvHelper._check_env_exists(EnvKeys.django_dev)

    @staticmethod
    def _check_env_exists(key: str) -> bool:
        if os.getenv(key, None) is None:
            return False
        return True

    @staticmethod
    def try_get_from_env(key: str, default: Optional[str] = None) -> Optional[str]:
        """
        从环境变量中获取配置 [有可能返回为 None]
        :param key: 环境变量的 key
        :param default: 默认值
        """
        return os.environ.get(key, default)

    @staticmethod
    def get_from_env(key: str, default: Optional[str] = None) -> str:
        """
        获取环境变量的值 [环境变量必须存在,否则退出进程]
        :param key: 环境变量的 key
        :param default: 默认值
        """
        value = EnvHelper.try_get_from_env(key, default)
        if value is not None:
            return value
        print(f"环境变量: {key} 没有设置, 启动服务器失败", file=sys.stderr)
        sys.exit(2)



from typing import Callable

from django.template.base import Parser
from django.template.context import Context

from .div_node import DivNode

__all__ = ["custom_div_node_helper"]


def custom_div_node_helper(name: str, parser: Parser, fn: Callable[[Context], bool]):
    """
    :param name: 节点的名称
    解析形式如下:

        {% $name %}
        # this sub content
        {% end_$name %}

    :param parser: django parser [pass as is]
    :param fn: 一个判断是否需要渲染子节点的函数
              这个函数的参数为: 模版的 Context
              True  -> 渲染子节点
              False -> 不需要渲染子节点

    usage demo:

        from django import template
        register = template.Library()

        @register.tag
        def custom_node_name(parser, token):
            def fn(ctx: Context) -> bool:
                return True
            return custom_div_node_helper("custom_node_name", parser, fn)

    """
    nodelist = parser.parse((f"end_{name}",))
    parser.delete_first_token()
    return DivNode(name, nodelist, fn)



from .paginator import CustomPaginator


def test_custom_page():
    p = CustomPaginator(range(10), 5)
    p1 = p.page(1)

    assert p1.show_page_start_index() == 1
    assert p1.show_page_have_previous_ellipse() is False
    assert p1.show_page_have_next_ellipse() is False
    assert p1.show_page_end_index() == 2

    p = CustomPaginator(range(11), 5)
    p1 = p.page(1)

    assert p1.show_page_start_index() == 1
    assert p1.show_page_have_previous_ellipse() is False
    assert p1.show_page_have_next_ellipse() is False
    assert p1.show_page_end_index() == 3

    p = CustomPaginator(range(20), 2)
    p1 = p.page(1)

    assert p1.show_page_start_index() == 1
    assert p1.show_page_have_previous_ellipse() is False
    assert p1.show_page_have_next_ellipse() is True
    assert p1.show_page_end_index() == 9
    assert list(p1.show_page_range()) == list(range(1, 10))

    p = CustomPaginator(range(20), 2)
    p1 = p.page(5)

    assert p1.show_page_start_index() == 1
    assert p1.show_page_have_previous_ellipse() is False
    assert p1.show_page_have_next_ellipse() is True
    assert p1.show_page_end_index() == 9
    assert list(p1.show_page_range()) == list(range(1, 10))

    p = CustomPaginator(range(20), 2)
    p1 = p.page(6)

    assert p1.show_page_start_index() == 2
    assert p1.show_page_have_previous_ellipse() is True
    assert p1.show_page_have_next_ellipse() is False
    assert p1.show_page_end_index() == 10
    assert list(p1.show_page_range()) == list(range(2, 11))



from typing import Iterable

from django.core.paginator import Paginator, Page

__all__ = ["CustomPaginator"]


class CustomPage(Page):
    max_show_pages = 9

    def show_page_start_index(self) -> int:
        assert isinstance(self.paginator, CustomPaginator)
        start = 1
        end = self.paginator.num_pages
        if end <= self.max_show_pages:
            return start
        if self.number - self._show_half_size() <= start:
            return start
        return self.number - self._show_half_size()

    def show_page_end_index(self) -> int:
        assert isinstance(self.paginator, CustomPaginator)
        real_end = self.paginator.num_pages
        expect_end = self.number + self._show_half_size()

        if real_end <= self.max_show_pages:  # 实际有的没有 需要展示的多
            return real_end

        if expect_end >= real_end:  # 期望的太多 超过实际有的
            return real_end

        if expect_end < self.max_show_pages:
            return self.max_show_pages

        return self.number + self._show_half_size()

    def show_page_have_previous_ellipse(self) -> bool:
        if self.number - self._show_half_size() > 1:
            return True
        return False

    def show_page_have_next_ellipse(self) -> bool:
        assert isinstance(self.paginator, CustomPaginator)
        if self.number + self._show_half_size() < self.paginator.num_pages:
            return True
        return False

    def show_page_range(self) -> Iterable[int]:
        return range(self.show_page_start_index(), self.show_page_end_index() + 1)

    def _show_half_size(self) -> int:
        return int(self.max_show_pages / 2)


class CustomPaginator(Paginator):
    def page(self, number) -> CustomPage:
        return super().page(number)

    def _get_page(self, *args, **kwargs) -> CustomPage:
        return CustomPage(*args, **kwargs)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['django_qiyu_utils', 'django_qiyu_utils.div_node']

package_data = \
{'': ['*']}

install_requires = \
['django>=3.1,<3.2']

setup_kwargs = {
    'name': 'django-qiyu-utils',
    'version': '0.3.5',
    'description': 'Django 辅助工具类',
    'long_description': '# 奇遇科技 Django 开发辅助工具箱\n\n![PyPI - Version](https://img.shields.io/pypi/v/django-qiyu-utils)\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/django-qiyu-utils)\n![PyPI - Downloads](https://img.shields.io/pypi/dm/django-qiyu-utils)\n![PyPI - Wheel](https://img.shields.io/pypi/wheel/django-qiyu-utils)\n![GitHub repo size](https://img.shields.io/github/repo-size/qiyutechdev/django-qiyu-utils)\n![Lines of code](https://img.shields.io/tokei/lines/github/qiyutechdev/django-qiyu-utils)\n\n# WARNING\n\n    USE IT AT YOUR OWN RISK!!\n',
    'author': 'dev',
    'author_email': 'dev@qiyutech.tech',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://www.qiyutech.tech/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



"""
从 HTTP Header 中获取的信息
"""
from typing import Optional

from django.http import HttpRequest

__all__ = ["get_client_ip"]


def get_client_ip(request: HttpRequest) -> Optional[str]:
    """
    获取用户的 IP 地址
    """

    x: str = request.META.get("HTTP_X_FORWARDED_FOR", None)
    if x:
        return x.split(",")[0]
    return request.META.get("REMOTE_ADDR", None)
